@@ -4,7 +4,7 @@
 * https://github.com/raforg/rawhide
 * https://codeberg.org/raforg/rawhide
 *
-# Copyright (C) 1990 Ken Stauffer, 2022 raf <raf@raf.org>
+# Copyright (C) 1990 Ken Stauffer, 2022-2023 raf <raf@raf.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <https://www.gnu.org/licenses/>.
 *
-* 20221011 raf <raf@raf.org>
+* 20230609 raf <raf@raf.org>
 */
 
 #define _GNU_SOURCE /* For FNM_EXTMATCH and FNM_CASEFOLD in <fnmatch.h> */
@@ -148,9 +148,21 @@ static void help_message(void)
 
 	printf("\nPattern modifiers:\n");
 
-	for (i = 0, s = symbols; s; s = s->next)
+	/* One column for each method of pattern matching */
+	int columns = 1;
+	#ifdef FNM_CASEFOLD
+	columns += 1; /* .i */
+	#endif
+	#ifdef HAVE_PCRE2
+	columns += 2; /* .re .rei */
+	#endif
+
+	if (columns > 1)
+		printf("  %-12s", "");
+
+	for (i = 1, s = symbols; s; s = s->next)
 		if (s->type == PATMOD)
-			printf("  %-12s%s", s->name, (i++ % 5 == 4 || !s->next) ? "\n" : "");
+			printf("  %-12s%s", s->name, (i++ % columns == columns - 1 || !s->next) ? "\n" : "");
 
 	#ifdef FNM_CASEFOLD
 	printf("  Case-insensitive glob matching is available here (i)\n");
@@ -160,6 +172,9 @@ static void help_message(void)
 	#else
 	printf("  Perl-compatible regular expressions are not available here\n");
 	#endif
+	#ifdef HAVE_MAGIC
+	printf("  File types and MIME types are available here (what, mime)\n");
+	#endif
 	#ifdef HAVE_ACL
 	printf("  Access control lists are available here (acl)\n");
 	#endif
@@ -192,7 +207,7 @@ static void help_message(void)
 	printf("GIT: https://github.org/raforg/rawhide\n");
 	printf("GIT: https://codeberg.org/raforg/rawhide\n");
 	printf("\n");
-	printf("Copyright (C) 1990 Ken Stauffer, 2022 raf <raf@raf.org>\n");
+	printf("Copyright (C) 1990 Ken Stauffer, 2022-2023 raf <raf@raf.org>\n");
 	printf("\n");
 	printf("This is free software released under the terms of the GPLv3+:\n");
 	printf("\n");
@@ -367,6 +382,44 @@ static int load_config_dir(const char *initfile, char *initdir, char *initpatter
 
 /*
 
+static void magic_cleanup(void);
+
+Release any libmagic resources that have been allocated.
+To be run at exit.
+
+*/
+
+#ifdef HAVE_MAGIC
+static void magic_cleanup(void)
+{
+	if (attr.what_cookie)
+	{
+		magic_close(attr.what_cookie);
+		attr.what_cookie = NULL;
+	}
+
+	if (attr.mime_cookie)
+	{
+		magic_close(attr.mime_cookie);
+		attr.mime_cookie = NULL;
+	}
+
+	if (attr.what_follow_cookie)
+	{
+		magic_close(attr.what_follow_cookie);
+		attr.what_follow_cookie = NULL;
+	}
+
+	if (attr.mime_follow_cookie)
+	{
+		magic_close(attr.mime_follow_cookie);
+		attr.mime_follow_cookie = NULL;
+	}
+}
+#endif
+
+/*
+
 int main(int argc, char *argv[]);
 
 Parse the command line:
@@ -1226,6 +1279,12 @@ int main(int argc, char *argv[])
 	if (!(attr.search_stack = (point_t *)calloc(attr.max_depth + 1, sizeof(point_t))))
 		fatal("out of memory");
 
+	/* Prepare to deallocate libmagic resources */
+
+	#ifdef HAVE_MAGIC
+	atexit(magic_cleanup);
+	#endif
+
 	/* Find matches in the given directories (or the current working directory) */
 
 	for (; optind < argc; optind++)
