@@ -4,7 +4,7 @@
 * https://github.com/raforg/rawhide
 * https://codeberg.org/raforg/rawhide
 *
-* Copyright (C) 1990 Ken Stauffer, 2022 raf <raf@raf.org>
+* Copyright (C) 1990 Ken Stauffer, 2022-2023 raf <raf@raf.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <https://www.gnu.org/licenses/>.
 *
-* 20221011 raf <raf@raf.org>
+* 20230609 raf <raf@raf.org>
 */
 
 #define _GNU_SOURCE /* For FNM_EXTMATCH and FNM_CASEFOLD in <fnmatch.h> */
@@ -227,13 +227,15 @@ void c_exit(llong i)    { Stack[SP++] = 1; attr.exit = 1; }
 
 unsigned long get_attr(void)
 {
-	#ifdef HAVE_ATTR
 	if (!attr.attr_done)
 	{
 		attr.attr_done = 1;
+		#if HAVE_ATTR
 		fgetflags(attr.fpath, &attr.attr);
+		#elif HAVE_FLAGS
+		attr.attr = (unsigned long)attr.statbuf->st_flags;
+		#endif
 	}
-	#endif
 
 	return attr.attr;
 }
@@ -264,8 +266,10 @@ unsigned long get_gen(void)
 	return attr.gen;
 }
 
-#ifdef HAVE_ATTR
+#if HAVE_ATTR || HAVE_FLAGS
 void c_attr(llong i) { Stack[SP++] = get_attr(); }
+#endif
+#if HAVE_ATTR
 void c_proj(llong i) { Stack[SP++] = get_proj(); }
 void c_gen(llong i)  { Stack[SP++] = get_gen(); }
 #endif
@@ -393,8 +397,6 @@ static pcre2_code *pcre2_compile_checked(const char *pattern, uint32_t options)
 		fatal("invalid regex %s at offset %d: %s", ok(pattern), (int)error_offset, ok2((char *)error_buffer));
 	}
 
-	pcre2_jit_compile(re, PCRE2_JIT_COMPLETE);
-
 	return re;
 }
 
@@ -500,10 +502,126 @@ void c_reilink(llong i) { Stack[SP++] = (islink(attr.statbuf)) ? rematch(&Strbuf
 
 #endif /* HAVE_PCRE2 */
 
-#define failure(args) if (want) { errorsys args; attr.exit_status = EXIT_FAILURE; }
+/* Load the file type description into attr.what (libmagic-managed data - deallocated at exit) */
+
+const char *get_what(void)
+{
+	if (!attr.what_done)
+	{
+		#ifdef HAVE_MAGIC
+		if (following_symlinks())
+		{
+			if (!attr.what_follow_cookie)
+				if ((attr.what_follow_cookie = magic_open(MAGIC_RAW | MAGIC_COMPRESS | MAGIC_SYMLINK)))
+					(void)magic_load(attr.what_follow_cookie, NULL);
+
+			if (attr.what_follow_cookie)
+				attr.what = magic_file(attr.what_follow_cookie, attr.fpath);
+		}
+		else
+		{
+			if (!attr.what_cookie)
+				if ((attr.what_cookie = magic_open(MAGIC_RAW | MAGIC_COMPRESS)))
+					(void)magic_load(attr.what_cookie, NULL);
+
+			if (attr.what_cookie)
+				attr.what = magic_file(attr.what_cookie, attr.fpath);
+		}
+		#endif
+
+		attr.what_done = 1;
+	}
+
+	return attr.what;
+}
+
+#ifdef HAVE_MAGIC
+
+static void what_glob(llong i, int options)
+{
+	Stack[SP++] = get_what() ? fnmatch(&Strbuf[i], get_what(), FNM_EXTMATCH | options) == 0 : 0;
+}
+
+void c_what(llong i)  { what_glob(i, 0); }
+#ifdef FNM_CASEFOLD
+void c_iwhat(llong i) { what_glob(i, FNM_CASEFOLD); }
+#endif
+
+#ifdef HAVE_PCRE2
+
+static void what_re(llong i, int options)
+{
+	Stack[SP++] = get_what() ? rematch(&Strbuf[i], get_what(), options) : 0;
+}
+
+void c_rewhat(llong i)  { what_re(i, 0); }
+void c_reiwhat(llong i) { what_re(i, PCRE2_CASELESS); }
+
+#endif
+#endif
+
+/* Load the MIME type into attr.what (libmagic-managed data - deallocated at exit) */
+
+const char *get_mime(void)
+{
+	if (!attr.mime_done)
+	{
+		#ifdef HAVE_MAGIC
+		if (following_symlinks())
+		{
+			if (!attr.mime_follow_cookie)
+				if ((attr.mime_follow_cookie = magic_open(MAGIC_MIME | MAGIC_SYMLINK)))
+					(void)magic_load(attr.mime_follow_cookie, NULL);
+
+			if (attr.mime_follow_cookie)
+				attr.mime = magic_file(attr.mime_follow_cookie, attr.fpath);
+		}
+		else
+		{
+			if (!attr.mime_cookie)
+				if ((attr.mime_cookie = magic_open(MAGIC_MIME)))
+					(void)magic_load(attr.mime_cookie, NULL);
+
+			if (attr.mime_cookie)
+				attr.mime = magic_file(attr.mime_cookie, attr.fpath);
+		}
+		#endif
+
+		attr.mime_done = 1;
+	}
+
+	return attr.mime;
+}
+
+#ifdef HAVE_MAGIC
+
+static void mime_glob(llong i, int options)
+{
+	Stack[SP++] = get_mime() ? fnmatch(&Strbuf[i], get_mime(), FNM_EXTMATCH | options) == 0 : 0;
+}
+
+void c_mime(llong i)  { mime_glob(i, 0); }
+#ifdef FNM_CASEFOLD
+void c_imime(llong i) { mime_glob(i, FNM_CASEFOLD); }
+#endif
+
+#ifdef HAVE_PCRE2
+
+static void mime_re(llong i, int options)
+{
+	Stack[SP++] = get_mime() ? rematch(&Strbuf[i], get_mime(), options) : 0;
+}
+
+void c_remime(llong i)  { mime_re(i, 0); }
+void c_reimime(llong i) { mime_re(i, PCRE2_CASELESS); }
+
+#endif
+#endif
 
 /* Load ACLs into attr.facl (and attr.facl_verbose on FreeBSD/Solaris) which must be deallocated */
 
+#define failure(args) if (want) { errorsys args; attr.exit_status = EXIT_FAILURE; }
+
 char *get_acl(int want)
 {
 	if (!attr.facl_done)
@@ -1286,7 +1404,7 @@ void r_ctime(llong i)   { check_reference(i, "ctime");   Stack[SP++] = RefFile[i
 void r_type(llong i)    { check_reference(i, "type");    Stack[SP++] = RefFile[i].statbuf->st_mode & S_IFMT; }
 void r_perm(llong i)    { check_reference(i, "perm");    Stack[SP++] = RefFile[i].statbuf->st_mode & ~S_IFMT; }
 
-#ifdef HAVE_ATTR
+#if HAVE_ATTR
 
 void r_attr(llong i)
 {
@@ -1327,6 +1445,21 @@ void r_gen(llong i)
 	Stack[SP++] = RefFile[i].gen;
 }
 
+#elif HAVE_FLAGS
+
+void r_attr(llong i)
+{
+	check_reference(i, "attr");
+
+	if (!RefFile[i].attr_done)
+	{
+		RefFile[i].attr_done = 1;
+		RefFile[i].attr = RefFile[i].statbuf->st_flags;
+	}
+
+	Stack[SP++] = RefFile[i].attr;
+}
+
 #endif
 
 /* Symlink targets */
@@ -1428,8 +1561,10 @@ char *instruction_name(void (*func)(llong))
 		(func == c_atime) ? "atime" :
 		(func == c_mtime) ? "mtime" :
 		(func == c_ctime) ? "ctime" :
-		#ifdef HAVE_ATTR
+		#if HAVE_ATTR || HAVE_FLAGS
 		(func == c_attr) ? "cattr" :
+		#endif
+		#if HAVE_ATTR
 		(func == c_proj) ? "cproj" :
 		(func == c_gen) ? "cgen" :
 		#endif
@@ -1459,6 +1594,26 @@ char *instruction_name(void (*func)(llong))
 		(func == c_reipath) ? "reipath" :
 		(func == c_reilink) ? "reilink" :
 		#endif
+		#ifdef HAVE_MAGIC
+		(func == c_what) ? "what" :
+		#ifdef FNM_CASEFOLD
+		(func == c_iwhat) ? "iwhat" :
+		#endif
+		#ifdef HAVE_PCRE2
+		(func == c_rewhat) ? "rewhat" :
+		(func == c_reiwhat) ? "reiwhat" :
+		#endif
+		#endif
+		#ifdef HAVE_MAGIC
+		(func == c_mime) ? "mime" :
+		#ifdef FNM_CASEFOLD
+		(func == c_imime) ? "imime" :
+		#endif
+		#ifdef HAVE_PCRE2
+		(func == c_remime) ? "remime" :
+		(func == c_reimime) ? "reimime" :
+		#endif
+		#endif
 		#ifdef HAVE_ACL
 		(func == c_acl) ? "acl" :
 		#ifdef FNM_CASEFOLD
@@ -1498,8 +1653,10 @@ char *instruction_name(void (*func)(llong))
 		(func == r_atime) ? "ratime" :
 		(func == r_mtime) ? "rmtime" :
 		(func == r_ctime) ? "rctime" :
-		#ifdef HAVE_ATTR
+		#if HAVE_ATTR || HAVE_FLAGS
 		(func == r_attr) ? "rattr" :
+		#endif
+		#if HAVE_ATTR
 		(func == r_proj) ? "rproj" :
 		(func == r_gen) ? "rgen" :
 		#endif
