@@ -4,7 +4,7 @@
 * https://github.com/raforg/rawhide
 * https://codeberg.org/raforg/rawhide
 *
-* Copyright (C) 1990 Ken Stauffer, 2022 raf <raf@raf.org>
+* Copyright (C) 1990 Ken Stauffer, 2022-2023 raf <raf@raf.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <https://www.gnu.org/licenses/>.
 *
-* 20221011 raf <raf@raf.org>
+* 20230609 raf <raf@raf.org>
 */
 
 #ifndef RAWHIDE_RHCMDS_H
@@ -70,8 +70,10 @@ void c_ctime(llong i);
 unsigned long get_attr(void);
 unsigned long get_proj(void);
 unsigned long get_gen(void);
-#ifdef HAVE_ATTR
+#if HAVE_ATTR || HAVE_FLAGS
 void c_attr(llong i);
+#endif
+#if HAVE_ATTR
 void c_proj(llong i);
 void c_gen(llong i);
 #endif
@@ -102,6 +104,26 @@ void c_rei(llong i);
 void c_reipath(llong i);
 void c_reilink(llong i);
 #endif
+#ifdef HAVE_MAGIC
+void c_what(llong i);
+#ifdef FNM_CASEFOLD
+void c_iwhat(llong i);
+#endif
+#ifdef HAVE_PCRE2
+void c_rewhat(llong i);
+void c_reiwhat(llong i);
+#endif
+#endif
+#ifdef HAVE_MAGIC
+void c_mime(llong i);
+#ifdef FNM_CASEFOLD
+void c_imime(llong i);
+#endif
+#ifdef HAVE_PCRE2
+void c_remime(llong i);
+void c_reimime(llong i);
+#endif
+#endif
 #ifdef HAVE_ACL
 void c_acl(llong i);
 #ifdef FNM_CASEFOLD
@@ -142,8 +164,10 @@ void r_blocks(llong i);
 void r_atime(llong i);
 void r_mtime(llong i);
 void r_ctime(llong i);
-#ifdef HAVE_ATTR
+#if HAVE_ATTR || HAVE_FLAGS
 void r_attr(llong i);
+#endif
+#if HAVE_ATTR
 void r_proj(llong i);
 void r_gen(llong i);
 #endif
@@ -173,6 +197,8 @@ void t_strlen(llong i);
 
 char *read_symlink(void);
 void prepare_target(void);
+const char *get_what(void);
+const char *get_mime(void);
 char *get_acl(int);
 char *get_ea(int);
 void set_dirsize(void);
