@@ -4,7 +4,7 @@
 * https://github.com/raforg/rawhide
 * https://codeberg.org/raforg/rawhide
 *
-* Copyright (C) 1990 Ken Stauffer, 2022 raf <raf@raf.org>
+* Copyright (C) 1990 Ken Stauffer, 2022-2023 raf <raf@raf.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <https://www.gnu.org/licenses/>.
 *
-* 20221011 raf <raf@raf.org>
+* 20230609 raf <raf@raf.org>
 */
 
 #define _GNU_SOURCE /* For FNM_EXTMATCH and FNM_CASEFOLD in <fnmatch.h> and wcswidth() in <wchar.h> */
@@ -171,6 +171,8 @@ static void caches_init(void)
 	attr.ftarget_done = 0;
 	attr.linkstat_done = 0;
 	attr.linkdirsize_done = 0;
+	attr.what_done = 0;
+	attr.mime_done = 0;
 	attr.facl_done = 0;
 	attr.facl = NULL;
 	attr.facl_verbose = NULL;
@@ -1510,7 +1512,7 @@ int interpolate_command(const char *srccmd, char *command, int cmdbufsize)
 				f = strrchr(attr.fpath, '/');
 				f = (f == attr.fpath && !f[1]) ? attr.fpath : (f) ? f + 1 : attr.fpath;
 
-				/* While there's room for a null or a byte followed by a nul... */
+				/* While there's room for a nul or a byte followed by a nul... */
 
 				while (dst + ((*f) ? 1 : 0) - command < cmdbufsize)
 				{
@@ -1976,8 +1978,8 @@ static int add_field(char *buf, ssize_t sz, const char *name, const char *value)
 
 static char *attributes(void);
 
-Return the ext2-style file attributes as a space-separated
-list of attribute names.
+Return the ext2-style file attributes or BSD-style file flags
+as a space-separated list of attribute/flag names.
 
 */
 
@@ -1996,6 +1998,8 @@ static char *attributes(void)
 	if (!flags)
 		return buf;
 
+	#if HAVE_ATTR /* Linux */
+
 	#ifdef EXT2_SECRM_FL
 	if (flags & EXT2_SECRM_FL)
 		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "secrm");
@@ -2146,6 +2150,135 @@ static char *attributes(void)
 		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "casefold");
 	#endif
 
+	#elif HAVE_FLAGS /* FreeBSD, OpenBSD, NetBSD, macOS */
+
+	#ifdef UF_NODUMP /* FreeBSD, OpenBSD, NetBSD, macOS */
+	if (flags & UF_NODUMP)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "nodump");
+	#endif
+
+	#ifdef UF_UF_IMMUTABLE /* FreeBSD, OpenBSD, NetBSD, macOS */
+	if (flags & UF_IMMUTABLE)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "uimmutable");
+	#endif
+
+	#ifdef UF_APPEND /* FreeBSD, OpenBSD, NetBSD, macOS */
+	if (flags & UF_APPEND)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "uappend");
+	#endif
+
+	#ifdef UF_OPAQUE /* FreeBSD, OpenBSD, NetBSD, macOS */
+	if (flags & UF_OPAQUE)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "opaque");
+	#endif
+
+	#ifdef UF_NOUNLINK /* FreeBSD = 0x10 */
+	if (flags & UF_NOUNLINK)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "unounlink");
+	#endif
+
+	#ifdef UF_COMPRESSED /* macOS = 0x20 */
+	if (flags & UF_COMPRESSED)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "compressed");
+	#endif
+
+	#ifdef UF_TRACKED /* macOS = 0x40 */
+	if (flags & UF_TRACKED)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "tracked");
+	#endif
+
+	#ifdef UF_SYSTEM /* FreeBSD = 0x80 */
+	if (flags & UF_SYSTEM)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "system");
+	#endif
+
+	#ifdef UF_DATAVAULT /* macOS = 0x80 */
+	if (flags & UF_DATAVAULT)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "datavault");
+	#endif
+
+	#ifdef UF_SPARSE /* FreeBSD = 0x100 */
+	if (flags & UF_SPARSE)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "sparse");
+	#endif
+
+	#ifdef UF_OFFLINE /* FreeBSD = 0x200 */
+	if (flags & UF_OFFLINE)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "offline");
+	#endif
+
+	#ifdef UF_REPARSE /* FreeBSD = 0x400 */
+	if (flags & UF_REPARSE)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "reparse");
+	#endif
+
+	#ifdef UF_ARCHIVE /* FreeBSD = 0x800 */
+	if (flags & UF_ARCHIVE)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "archive");
+	#endif
+
+	#ifdef UF_READONLY /* FreeBSD = 0x1000 */
+	if (flags & UF_READONLY)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "readonly");
+	#endif
+
+	#ifdef UF_HIDDEN /* FreeBSD, macOS = 0x8000 */
+	if (flags & UF_HIDDEN)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "hidden");
+	#endif
+
+	#ifdef SF_ARCHIVED /* FreeBSD, OpenBSD, macOS = 0x10000 */
+	if (flags & SF_ARCHIVED)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "archived");
+	#endif
+
+	#ifdef SF_IMMUTABLE /* FreeBSD, OpenBSD, macOS = 0x20000 */
+	if (flags & SF_IMMUTABLE)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "simmutable");
+	#endif
+
+	#ifdef SF_APPEND /* FreeBSD, OpenBSD, macOS = 0x40000 */
+	if (flags & SF_APPEND)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "sappend");
+	#endif
+
+	#ifdef SF_RESTRICTED /* macOS = 0x80000 */
+	if (flags & SF_RESTRICTED)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "restricted");
+	#endif
+
+	#ifdef SF_NOUNLINK /* FreeBSD, macOS = 0x100000 */
+	if (flags & SF_NOUNLINK)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "snounlink");
+	#endif
+
+	#ifdef SF_SNAPSHOT /* FreeBSD, NetBSD = 0x200000 */
+	if (flags & SF_SNAPSHOT)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "snapshot");
+	#endif
+
+	#ifdef SF_LOG /* NetBSD = 0x400000 */
+	if (flags & SF_LOG)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "log");
+	#endif
+
+	#ifdef SF_SNAPINVAL /* NetBSD = 0x800000 */
+	if (flags & SF_SNAPINVAL)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "snapinval");
+	#endif
+
+	#ifdef SF_FIRMLINK /* macOS = 0x800000 */
+	if (flags & SF_FIRMLINK)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "firmlink");
+	#endif
+
+	#ifdef SF_DATALESS /* macOS = 0x40000000 */
+	if (flags & SF_DATALESS)
+		pos += ssnprintf(buf + pos, ATTR_BUFSIZE - pos, "%s%s", (pos) ? " " : "", "dataless");
+	#endif
+
+	#endif
+
 	return buf;
 }
 
@@ -2218,9 +2351,17 @@ static char *json(void)
 	pos += ssnprintf(buf + pos, JSON_BUFSIZE - pos, "\"mtime_unix\":%lld, ", (llong)attr.statbuf->st_mtime);
 	pos += ssnprintf(buf + pos, JSON_BUFSIZE - pos, "\"ctime_unix\":%lld, ", (llong)attr.statbuf->st_ctime);
 
-	#ifdef HAVE_ATTR
-	pos += ssnprintf(buf + pos, JSON_BUFSIZE - pos, "\"attributes\":\"%s\", ", attributes());
+	if (get_what())
+		pos += add_field(buf + pos, JSON_BUFSIZE - pos, "filetype", get_what());
 
+	if (get_mime())
+		pos += add_field(buf + pos, JSON_BUFSIZE - pos, "mimetype", get_mime());
+
+	#if HAVE_ATTR || HAVE_FLAGS
+	pos += ssnprintf(buf + pos, JSON_BUFSIZE - pos, "\"attributes\":\"%s\", ", attributes());
+	#endif
+
+	#if HAVE_ATTR
 	pos += ssnprintf(buf + pos, JSON_BUFSIZE - pos, "\"project\":%lu, ", get_proj());
 
 	pos += ssnprintf(buf + pos, JSON_BUFSIZE - pos, "\"generation\":%lu, ", get_gen());
@@ -2769,6 +2910,24 @@ void visitf_format(void)
 						break;
 					}
 
+					case 'w': /* File type description */
+					{
+						ofmt_add('s');
+						debug_extra(("fmt %%w \"%s\", \"%s\"", ofmt, get_what() ? attr.what : ""));
+						printf_sanitized(ofmt, get_what() ? attr.what : "");
+
+						break;
+					}
+
+					case 'W': /* MIME type */
+					{
+						ofmt_add('s');
+						debug_extra(("fmt %%W \"%s\", \"%s\"", ofmt, get_mime() ? attr.mime : ""));
+						printf_sanitized(ofmt, get_mime() ? attr.mime : "");
+
+						break;
+					}
+
 					case 'x': /* Extended attributes */ 
 					{
 						char *ea, *s;
@@ -2776,7 +2935,7 @@ void visitf_format(void)
 						if ((ea = get_ea(1)) && attr.fea_ok)
 						{
 							/* We have finished searching and can modify the buffer. */
-							/* Replace newlines with commas and remove the last one */
+							/* Replace newlines with commas and remove the last one. */
 
 							for (s = ea; *s; ++s)
 								if (*s == '\n')
@@ -2897,7 +3056,7 @@ void visitf_format(void)
 						break;
 					}
 
-					case 'e': /* Ext2-style file attributes */
+					case 'e': /* Ext2-style file attributes and BSD-style file flags */
 					{
 						char *a = attributes();
 
